## 简介

地图点位渲染基础

### [渲染点位接口](./src/interface/point_con.ts)

```ts

/** 点位加载数据 */
interface IPointOptions {
    /** x 坐标 */
    x?: number;
    /** y 坐标 */
    y?: number;
    /** 点位id 可以转换成 xy */
    pointId?: number;
    /** 当前点位的 1 级模型 */
    pointNameLod1?: string;
    /** 当前点位的 2 级模型 */
    pointNameLod2?: string;
    /** 分级模型显示的临界缩放，地图缩放小于该值时, 渲染 pointNameLod1，大于该值时，渲染 pointNameLod2  */
    scaleBoundary: number;
}

/** 点位更新数据 */
interface IRenderUpdateData {
    /** 位置数据 */
    position?: [number, number, number];
    /** 旋转数据 */
    rotation?: [number, number, number];
    /** 缩放数据 */
    scaling?: [number, number, number];
    /** 是否显示隐藏 */
    isEnabled?: boolean;
}
```

### [使用示例](./src/client/point_manager.ts)

```ts
class MapPointMgr {
    /** 点位id转换为坐标 */
    public static pointIdToPos(pointId: number): [number, number, number];

    /** 坐标转点位id */
    public static posToPointId(x: number, y: number): number;

    /**
     * 通过 起点id 和 终点id 渲染一条线
     */
    public static renderPath(progress: number, startId: number, endId: number): string;

    /**
     * 通过 起点坐标 和 终点坐标 渲染一条线
     * @param progress 线的进度，会改变起点
     */
    public static renderPath(progress: number, startx: number, starty: number, endx: number, endy: number): string;

    /**
     * 通过线id移除线
     */
    public static removePath(pathId: string): void;

    /**
     * 通过 起点id 和 终点id 移除
     */
    public static removePath(startId: number, endId: number): void;

    /**
     * 通过 起点坐标 和 终点坐标 移除
     */
    public static removePath(startx: number, starty: number, endx: number, endy: number): void;

    /**
     * 渲染一个点
     */
    public static renderPoint(pointOptions: IPointOptions): string[];

    /**
     * 更新一个点
     */
    public static updatePoint(pointOptions: IPointOptions, updateInfo: IRenderUpdateData): void;

    /**
     * 通过 id 移除点位
     */
    public static removePoint(pointId: number): void;

    /**
     * 通过 坐标 移除点
     */
    public static removePoint(x: number, y: number): void;
}

```

### [进度条使用使用案例](./src/client/render/progress/custom_progress_control.ts)

```ts

/** 自定义进度条创建参数 */
interface IProgressOptions {
    /** 自定义名称 */
    progressnName: string;
    /** 公告板模式 */
    billboardMode: number;
    /** 父节点 */
    parent?: BABYLON.TransformNode;
    /** 缩放 */
    scaling?: [number, number, number];
    /** 位置 */
    position?: [number, number, number];
    /** 旋转 */
    rotation?: [number, number, number];
}

class CustomProgressControl {
    /**
     * 在指定场景创建一个进度条，需要传入纹理路径
     */
    public static instancePorgressMesh(scene: SceneStruct, progressnOptions: IProgressOptions, path: string): BABYLON.InstancedMesh;

    /** 更新进度条的进度 */
    public static changeProgress(progressMesh: BABYLON.InstancedMesh, progressNum: number)
}

```
