export class RenderCallBack<T> {
    public render: (arg?: T) => void;
    public readonly clear: () => void;
    public readonly remove: (fn: (arg?: T) => void) => void;

    constructor() {
        return RenderCallBack.createdProxy(this);
    }

    private static createdProxy = <T>(example: RenderCallBack<T>) => {
        return new Proxy(example, {
            get(target, method: string, receiver) {
                // 默认为 render 执行
                let executor = (...args) => {
                    let list: Function[] = Reflect.get(target, "_renderList", receiver) || [];
                    if (!list.length) return;
                    list.forEach(cb => cb(...args));
                }
                if (method === 'remove') {
                    // 设置为移除
                    executor = (cb) => {
                        let list: Function[] = Reflect.get(target, "_renderList", receiver) || [];
                        if (!list.includes(cb)) return false;
                        list.splice(list.indexOf(cb), 1);
                        return true;
                    }
                } 
                else if (method === 'clear') {
                    // 设置为清空
                    executor = () => {
                        Object.keys(target).forEach(key => {
                            Reflect.deleteProperty(target, key);
                        });
                        console.log('clear callbacks--->', target);
                        return true
                    }
                }
                return executor;
            },
            set(target, method: string, cb: () => void, receiver) {
                if (method !== "render") return false; // 只有 render 方法可写
                let list: Function[] = Reflect.get(target, "_renderList", receiver) || [];
                if (list.includes(cb)) return false
                list.push(cb);
                return Reflect.set(target, "_renderList", list, receiver);
            }
        })
    }
}

export class UpdateCallBack<T> {
    public update: (arg?: T) => void;
    public readonly clear: () => void;
    public readonly remove: (fn: (arg?: T) => void) => void;

    constructor() {
        return UpdateCallBack.createdProxy(this);
    }

    private static createdProxy = <T>(example: UpdateCallBack<T>) => {
        return new Proxy(example, {
            get(target, method: string, receiver) {
                // 默认为 update 执行
                let executor = (...args) => {
                    let list: Function[] = Reflect.get(target, "_updateList", receiver) || [];
                    if (!list.length) return;
                    list.forEach(cb => cb(...args));
                }
                if (method === 'remove') {
                    // 设置为移除
                    executor = (cb) => {
                        let list: Function[] = Reflect.get(target, "_updateList", receiver) || [];
                        if (!list.includes(cb)) return false;
                        list.splice(list.indexOf(cb), 1);
                        return true;
                    }
                } 
                else if (method === 'clear') {
                    // 设置为清空
                    executor = () => {
                        Object.keys(target).forEach(key => {
                            Reflect.deleteProperty(target, key);
                        });
                        console.log('clear callbacks--->', target);
                        return true
                    }
                }
                return executor;
            },
            set(target, method: string, cb: () => void, receiver) {
                if (method !== "update") return false; // 只有 update 方法可写
                let list: Function[] = Reflect.get(target, "_updateList", receiver) || [];
                if (list.includes(cb)) return false
                list.push(cb);
                return Reflect.set(target, "_updateList", list, receiver);
            }
        })
    }
}