import { SceneStruct } from "pi_babylon/src/scene_base";
import { MapRenderScene } from "./render/map_scene";
import { CityMapManager } from "./render/map_manager";
import { MapMouseControl } from "./logic/map_mouse_control";
import { SmallMap } from "pi_map/xls/small_map/map.struct";
import { getSafeLayoutSize } from "game_root/client/root";
import { BABYLON } from "pi_babylon/render3d/babylon";



export class CityMap {
    //#region 城市地图控制器
    private static _cityMapManager: CityMapManager;
    public static getCityManager(): CityMapManager {
        return this._cityMapManager;
    }
    //#初始化
    public static init(scene: SceneStruct, scale:number) {
        const cityRenderScene = new MapRenderScene(scale, scene);
        const mouseControl = new MapMouseControl(cityRenderScene);
        const cityMapManager = new CityMapManager(mouseControl, cityRenderScene, scale);

        this._cityMapManager = cityMapManager;
    }
    /**
     * 3d坐标转2d坐标
     */
    public static u3dTo2dPos(x:number, y:number) {
        let camera: BABYLON.Camera = this.getCityManager().mapRenderScene.camera;
        return {
            left: Math.floor((x - camera.orthoLeft - camera.position.x) / (camera.orthoRight - camera.orthoLeft) * getSafeLayoutSize().width),
            top: Math.floor((1 - (y - camera.orthoBottom - camera.position.y) / (camera.orthoTop - camera.orthoBottom)) * getSafeLayoutSize().height),
        };
    }
    
    /**
     * 2d坐标转3d
     */
    public static u2dTo3dPos(x:number, y:number) {
        // // 屏幕xy > 特效所在 gui canvas 节点 xy > 归约到 0~1 > y = 1 - y > (x * (orthRight - orthLeft) + orthLeft, y * (orthTop - orthBottom) + orthBottom)
        // let camera: BABYLON.Camera = this.getCityManager().mapRenderScene.camera;
        // x = x / getSafeLayoutSize().width;
        // y = 1 - (y / getSafeLayoutSize().height);
        // return {
        //     x:  Math.floor(x * (camera.orthoRight - camera.orthoLeft) + camera.orthoLeft),
        //     y:  Math.floor(y * (camera.orthoTop - camera.orthoBottom) + camera.orthoBottom),
        // };
    }
    //更新场景背景
    public static updateBackground(cfg: SmallMap, scale:number, scaleScope:[number, number]) {
        this.getCityManager().setCamera(scale, scaleScope, cfg.width, cfg.height);
    }
}