import { BABYLON } from "pi_babylon/render3d/babylon";
import { SceneStruct } from "pi_babylon/scene_base";
import { FormatEvent } from "pi_gui/engine/format_event";
import { getDocument } from "pi_gui/main";
import { IRayCastInfo } from "pi_babylon/render3d/scene";
import { TTransEventData } from "../../interface/scene_con";
import { AdaptationManage } from "../../client/logic/adaptation";
import { IScene3DEventInfo } from "../../interface/screen_con";
import { getContentWH } from "pi_babylon/index";
import { PISYS } from "pi_sys/index";

/**
 * 事件数据转换
 */
const transEvent = (e: TTransEventData) => {
    let pixelRatio = AdaptationManage.getScreenInfo().pixelRatio;

    const uiEvent = e;
    let sceneEvent: any = {};

    if (e.sourceEvent) { // 有原生事件说明是ui事件
        let info = getContentWH();
        const formatEvent = new FormatEvent(null);
        (<any>formatEvent).recordEventAttr(e.sourceEvent, sceneEvent);
        let clientX = e.sourceEvent.clientX;
        let clientY = e.sourceEvent.clientY;
        sceneEvent.clientX = (clientX - info.styleLeft) * pixelRatio;
        sceneEvent.clientY = (clientY - info.styleTop) * pixelRatio;
        sceneEvent.x = sceneEvent.clientX;
        sceneEvent.y = sceneEvent.clientY;
    }
    else {
        // e.clientX *= pixelRatio;
        // e.clientY *= pixelRatio;
        // e.x *= pixelRatio;
        // e.y *= pixelRatio;
        sceneEvent = e;
    }

    return { uiEvent, sceneEvent };
}

/**
 * 事件数据转场景命中
 */
export const formatEventDataToScene = (sceneStruct: SceneStruct, e: TTransEventData): IScene3DEventInfo => {
    let { sceneEvent, uiEvent } = transEvent(e);
    let _a;

    let pickRes = sceneStruct.impl.pick(sceneEvent.x, sceneEvent.y, mesh => {
        return mesh.isPickable && mesh.isEnabled();
    }, true);

    // pickRes = sceneStruct.impl.pickWithBoundingInfo(trans.x, trans.y, mesh => {
    //     return mesh.isPickable && mesh.isEnabled();
    // }, true);
    // console.log(pickRes);

    (<IRayCastInfo>pickRes).target = pickRes.pickedMesh;
    (<IRayCastInfo>pickRes).rayID = (<IRayCastInfo>pickRes).target ? (_a = (<IRayCastInfo>pickRes).target) === null || _a === void 0 ? void 0 : _a.rayID : null;

    const rayInfo = <IRayCastInfo>pickRes;

    return { rayInfo, sceneEvent, isSendNextLayer: e.isSendNextLayer, uiEvent };
}

/**
 * 查询点击时命中指定平面的点信息
 * @param cameraName 相机
 * @param scene 场景
 * @param x 事件坐标 - 在canvas范围内的坐标
 * @param y 事件坐标 - 在canvas范围内的坐标
 * @param plane 平面: y=0 的 x-z 平面则为 new BABYLON.Plane(0, 1, 0, 0)
 */
export const rayIntersectsPlane = (camera: BABYLON.Camera, scene: SceneStruct, x: number, y: number, plane: BABYLON.Plane): BABYLON.Vector3 | undefined => {
    if (!scene || !camera || !x || !y) return;
    let pickedPoint: BABYLON.Vector3 | undefined;
    const ray = scene.impl.createPickingRay(x, y, BABYLON.Matrix.Identity(), camera);
    const distance = ray.intersectsPlane(plane);
    // Get picked point
    const world = BABYLON.Matrix.Identity(); // this.getWorldMatrix();
    const worldOrigin = BABYLON.Vector3.Zero();
    const direction = BABYLON.Vector3.Zero();
    BABYLON.Vector3.TransformCoordinatesToRef(ray.origin, world, worldOrigin);
    ray.direction.scaleToRef(distance, direction);
    const worldDirection = BABYLON.Vector3.TransformNormal(direction, world);
    pickedPoint = worldDirection.addInPlace(worldOrigin);

    return pickedPoint;
}