import { IPITextureInfo } from "pi_babylon/material_info";
import { BABYLON } from "pi_babylon/render3d/babylon";
import { ShaderMaterial } from "pi_babylon/material/shader_material";
import { LoadExtraMaterial } from "pi_babylon/render3d/extends/material";
const KEY_AREA = "AreaMaterial";
//laserEffect
BABYLON.Effect.ShadersStore[`${KEY_AREA}VertexShader`] = `
    precision mediump float;

    attribute vec3 position;
    attribute vec4 color;
    attribute vec2 uv;

    uniform mat4 worldViewProjection;
    uniform vec4 bgTilloff;

    varying vec4 vColor;
    varying vec2 vBGUV;
    varying vec2 vUV;
    varying vec2 vScreen;

    void main(void) {
        vec3 positionUpdated = position;
        vec2 uvUpdated = uv;

        gl_Position = worldViewProjection*vec4(positionUpdated, 1.0);

        vUV = uv;
        vBGUV = bgTilloff.xy * uv + bgTilloff.zw;

        vColor = color;
        vScreen = gl_Position.xy * 0.5 + 0.5;
        vScreen.y = 1.0 - vScreen.y;
    }
`;

BABYLON.Effect.ShadersStore[`${KEY_AREA}FragmentShader`] =
    `
    precision mediump float;

    varying vec4 vColor;
    varying vec2 vBGUV;
    varying vec2 vUV;
    varying vec2 vScreen;

    uniform sampler2D repeatBGTexture;
    uniform sampler2D gradianBTexture;
    uniform sampler2D maskTexture;

    void main(void) {
        vec4 bgcol  = texture2D(repeatBGTexture, vBGUV);
        vec4 gradB  = texture2D(gradianBTexture, vScreen);
        vec4 mask   = texture2D(maskTexture, vUV);
        float v = mask.a;

        vec4 grad      = (gradB * gradB.a) * (1.0 - v);
        gl_FragColor   = vColor * 0. +  (grad * grad.a +  bgcol * (1.0 - grad.a));
        gl_FragColor.a = 1.;
    }
`;

export class AreaMaterial extends ShaderMaterial {
    public static readonly KEY = KEY_AREA;

    private _time: number = Date.now();
    public _tilloff: BABYLON.Vector4 = new BABYLON.Vector4(1, 1, 0, 0);
    public get tilloff() {
        return this._tilloff;
    }
    public set tilloff(value: BABYLON.Vector4) {
        this._tilloff = value;
    }
    constructor(materialName: string, babylonScene: any) {
        super(
            materialName,
            babylonScene,
            `${AreaMaterial.KEY}`,
            {
                attributes: ["position", "uv", "color"],
                uniforms: ["worldViewProjection"],
                samplers: ["repeatBGTexture", "gradianBTexture", "maskTexture"],
                needAlphaBlending: true
            }
        );
        this.setVector4("bgTilloff", this._tilloff);
    }
    protected _repeatBGTexture: BABYLON.Texture;
    public set repeatBGTexture(tex: BABYLON.Texture) {
        this.setTexture('repeatBGTexture', tex);
        this._repeatBGTexture = tex;
    }
    public get repeatBGTexture() {
        return this._repeatBGTexture;
    }
    
    protected _gradianBTexture: BABYLON.Texture;
    public set gradianBTexture(tex: BABYLON.Texture) {
        this.setTexture('gradianBTexture', tex);
        this._gradianBTexture = tex;
    }
    public get gradianBTexture() {
        return this._gradianBTexture;
    }

    protected _maskTexture: BABYLON.Texture;
    public set maskTexture(tex: BABYLON.Texture) {
        this.setTexture('maskTexture', tex);
        this._maskTexture = tex;
    }
    public get maskTexture() {
        return this._maskTexture;
    }

    isReady(mesh?: BABYLON.AbstractMesh, useInstances?: boolean): boolean {
        let result = !!this._repeatBGTexture && !!this._gradianBTexture && !!this._maskTexture;
        return result && super.isReady(mesh, useInstances);
    }
    bind(world: BABYLON.Matrix, mesh?: BABYLON.Mesh, effectOverride?: BABYLON.Effect): void {
        super.bind(world, mesh, effectOverride);
    }
    dispose(forceDisposeEffect?: boolean, forceDisposeTextures?: boolean, notBoundToMesh?: boolean): void {
        this._repeatBGTexture?.dispose();
        this._gradianBTexture?.dispose();
        this._maskTexture?.dispose();
        super.dispose(forceDisposeEffect, forceDisposeTextures, notBoundToMesh);
    }
}