import { ILine } from "pi_babylon/line/interface";
import { BABYLON } from "pi_babylon/render3d/babylon";
import { LineRenderer2D } from "pi_babylon/line/line";
import { SceneStruct } from "pi_babylon/scene_base";
import { Constants, CustomGeometry } from "pi_babylon/index";

interface MapCfg{
    id: number,
    name: string,
    width: number,//地图宽
    height: number,//地图宽
    path: string,//地图背景完整路径 | Psd内的图片名
}

//场景背景
export class SceneBgMgr {
    constructor(private scene:SceneStruct){

    }
    //地图配置
    private mapCfg: MapCfg;
    public getBgCfg() {
        return this.mapCfg;
    }
    //背景面片,使用线的方式平铺
    public background: ILine;
    public screenGraduation: BABYLON.Mesh;
    public geometryGraduation: CustomGeometry;
    //改场景背景
    public updateBackground(cfg:MapCfg) {
        this.mapCfg = cfg;
        if(this.background){
            this.background.dispose();
        }
        let {width, height, path} = cfg;
        const line = new LineRenderer2D(this.scene, new BABYLON.Vector3(0, 0, 1), 128);
        line.alphaIndex = 1000;
        this.background = line.create(
            path,
            -width/2, 0, 0,
            width/2, 0, 0,
            1, 1, 1, 1,
            height / 2, 1024, height/1024
        );

        if (this.screenGraduation == undefined) {
            this.screenGraduation = new BABYLON.Mesh("BGGraduation", this.scene.impl);
            let geometry = new CustomGeometry(this.screenGraduation, "BGGraduation", [Constants.VERTEX_ATTR_POSITION, Constants.VERTEX_ATTR_COLOR4, Constants.VERTEX_ATTR_UV], 6, 2, true);
            this.geometryGraduation = geometry;
            let material = new BABYLON.StandardMaterial("BGGraduation", this.scene.impl);
            material.emissiveColor.copyFromFloats(1, 1, 1);
            material.disableLighting = true;
            material.needAlphaBlending = () => true;
            material.alphaMode = 2;
            this.screenGraduation.material = material;
            this.screenGraduation.parent = this.scene.camera;
            this.screenGraduation.alphaIndex = 1001;
            this.screenGraduation.position.z = 100
            this.screenGraduation.material.backFaceCulling = false
            this.screenGraduation.alwaysSelectAsActiveMesh = true
            this.scene.impl.onBeforeRenderObservable.add(() => {
                let camera: BABYLON.FreeCamera = this.scene.camera;
                this.screenGraduation.setScaling(Math.abs(camera.orthoRight - camera.orthoLeft), Math.abs(camera.orthoTop - camera.orthoBottom), 1)
            })
        }
        if (this.geometryGraduation) {
            let color1 = cfg.color1 || [83/255, 255/255, 197/255, 0.6];
            let color2 = cfg.color2 || [83/255, 255/255, 197/255, 0.02];
            let data = this.geometryGraduation.indices();
            data[0] = 0; data[1] = 1; data[2] = 2; data[3] = 0; data[4] = 2;data[5] = 3;
            let [data1, strip1, offset1] = this.geometryGraduation.vertices(Constants.VERTEX_ATTR_POSITION);
            data1[offset1 + 0 * 3 + 0] = -0.5; data1[offset1 + 0 * 3 + 1] = -0.5; data1[offset1 + 0 * 3 + 2] = 0;
            data1[offset1 + 1 * 3 + 0] =  0.5; data1[offset1 + 1 * 3 + 1] = -0.5; data1[offset1 + 1 * 3 + 2] = 0;
            data1[offset1 + 2 * 3 + 0] =  0.5; data1[offset1 + 2 * 3 + 1] =  0.5; data1[offset1 + 2 * 3 + 2] = 0;
            data1[offset1 + 3 * 3 + 0] = -0.5; data1[offset1 + 3 * 3 + 1] =  0.5; data1[offset1 + 3 * 3 + 2] = 0;
            let [data2, strip2, offset2] = this.geometryGraduation.vertices(Constants.VERTEX_ATTR_COLOR4);
            data2[offset2 + 0 * 4 + 0] = color1[0]; data2[offset2 + 0 * 4 + 1] = color1[1]; data2[offset2 + 0 * 4 + 2] = color1[2]; data2[offset2 + 0 * 4 + 3] = color1[3];
            data2[offset2 + 1 * 4 + 0] = color1[0]; data2[offset2 + 1 * 4 + 1] = color1[1]; data2[offset2 + 1 * 4 + 2] = color1[2]; data2[offset2 + 1 * 4 + 3] = color1[3];
            data2[offset2 + 2 * 4 + 0] = color2[0]; data2[offset2 + 2 * 4 + 1] = color2[1]; data2[offset2 + 2 * 4 + 2] = color2[2]; data2[offset2 + 2 * 4 + 3] = color2[3];
            data2[offset2 + 3 * 4 + 0] = color2[0]; data2[offset2 + 3 * 4 + 1] = color2[1]; data2[offset2 + 3 * 4 + 2] = color2[2]; data2[offset2 + 3 * 4 + 3] = color2[3];
            this.geometryGraduation.update(6);
        }
    }
}
