import { ModelPrefab } from "pi_babylon/index";
import { ModleBase, IBaseData } from "./model_base";
import { getHsb } from "./hsb";
import { BABYLON } from "pi_babylon/render3d/babylon";

export class PointEffect extends ModleBase {
    //unity特效
    public u3dList: ModelPrefab[] = [];
    //序列帧特效
    public effectList: ModelPrefab[] = [];
    public HSV_DISABLED:BABYLON.Color3 = new BABYLON.Color3(0, -1, 0);//灰色
    public HSV_NORMAL:BABYLON.Color3 = new BABYLON.Color3(0, 0, 0);//正常色

    public constructor(private data: EffectData) {
        super(data);
        this.createU3dList();
        this.createEffectList();
    }
    public createU3dList() {
        if(!this.data.u3dEffect){return}
        for(let effName of this.data.u3dEffect){
            let animaName = this.data.lock ? "02" : "01";
            let model = this.createU3dEffect(effName, animaName, this.data.scaling, this.data.alphaIndex);
            model.setPosition(...this.data.position);
            this.u3dList.push(model)
        }
    }
    public createEffectList() {
        if(!this.data.effectList){return}
        if(this.data.effectBg){
            let modelBg = this.createSprite(this.data.effectBg, "bahuang_0", this.data.scaling, this.data.alphaIndex);
            let color = this.data.lock ? this.HSV_DISABLED : this.HSV_NORMAL;
            modelBg.setColor(color.r, color.g, color.b);
            this.effectList.push(modelBg);
        }
        for(let [effName, hsb] of this.data.effectList){
            let name = effName === "eff_01" ? "bahuang_anima" : "bahuang_anima2";
            let model = this.createAnimaSprite(name, name, this.data.scaling, this.data.alphaIndex + 1);
            model.setAnim({
                isLoop: true,
                animName: effName,
                speed: 0.6,
            });
            //灰度图
            if(hsb && !this.data.lock){
                let color = getHsb(hsb);
                model.setColor(color.r, color.g, color.b);
                color = null;
            }
            //非灰度图
            if(!hsb && this.data.lock){
                model.setColor(0, -1, 0);
            }else{
                model.setColor(0, 0, 0);
            }
            this.effectList.push(model)
        }
    }
    public setParent(root) {
        this.Root.setParent(root);
    }
    public setScale(scale:number) {
        this.Root.setScale(scale, scale, scale);
    }
    //销毁
    public dispose() {
        this.u3dList.forEach(v=>v.dispose());
        this.effectList.forEach(v=>v.dispose());
        this.u3dList.length = 0;
        this.effectList.length = 0;
        super.dispose();
    }
}

export interface EffectData extends IBaseData {
    effectBg:string,//建筑特效背景
    effectList:string[],//建筑序列帧特效
    u3dEffect:string[],//建筑unity特效
    lock: boolean,//是否未解锁
    scaling:[number, number, number],//大小
}

