import { BABYLON } from "pi_babylon/render3d/babylon";
import { LineRenderer2D } from "pi_babylon/line/line";
import { ILine } from "pi_babylon/line/interface";
import { RES_PATH } from "game_map/xls/map/map.struct";



// export enum LineColor{
//     disbaled = new BABYLON.Color3(0, -1, 0),//置灰
//     normal = new BABYLON.Color3(0, -1, 0),//正常色
// }
/**
 * 线的数据
 */
export interface LineData {
    id:number,//线的id
    start_x: number,//起点x
    start_y: number,//起点y
    end_x: number,//终点x
    end_y: number,//终点y
    line: string,//线使用的图片名字
    width: number,
    height: number,
    // HSB?:BABYLON.Color3,//不传使用 LineColor.normal
}

/**
 * 线渲染和数据信息
 */
interface EMapLineItem {
    cfg: LineData,
    render: ILine,
}


export class MapLine{
    private Line:LineRenderer2D;
    public allLineMap:Map<number, EMapLineItem> = new Map();
    public constructor(scene, alphaIndex:number){
        // this.Line = new LineRenderer2D(scene, new BABYLON.Vector3(0, 0, 1), 128, 2);//使用HSB
        this.Line = new LineRenderer2D(scene, new BABYLON.Vector3(0, 0, 1), 1024, 0);
        this.Line.alphaIndex = alphaIndex;
    };

    public getLinePath(cfg:LineData){
        return `${RES_PATH + cfg.line}.png`;
    };
    //创建
    public create(cfg:LineData){
        // let HSB = cfg.HSB || LineColor.normal;
        const line = this.Line.create(
            this.getLinePath(cfg),
            cfg.start_x,  cfg.start_y, 0,
            cfg.end_x,  cfg.end_y, 0,
            // HSB.r, HSB.g, HSB.b, 1,
            1, 1, 1, 1,
            // CFG.line.height, cfg.line ? Math.sqrt(Math.pow(end_x - start_x, 2) + Math.pow(end_y - start_y, 2)) : CFG.line.width, 1
            cfg.height, cfg.width, 1
        );
        this.allLineMap.set(cfg.id, {
            render: line,
            cfg
        })
        return line;
    };
    //更新
    public update(data:LineData){
        this.dispose(data.id);
        const sceneLine:ILine = this.create(data);
        this.allLineMap.set(data.id, {
            render: sceneLine,
            cfg: data
        })
    };
    //获取元素
    public getRenderIteam(id:number){
        return this.allLineMap.get(id)?.render;
    };
    //清理掉某一个
    public dispose(id:number){
        const item = this.allLineMap.get(id);
        if(!item){return}
        item.render.dispose();
        this.allLineMap.delete(id);
    };
    //清理掉所有
    public clear(){
        for(let [id, item] of this.allLineMap){
            item.render.dispose();
        }
        this.allLineMap.clear();
    };
}


