import { CustomScene, NodeObj, IPrefab } from "pi_babylon/index";
import { CreateImgPlane } from "game_scene_boom/create_img_plane";
import { RES_PATH } from "game_map/xls/map/map.struct";
import { CreateEffect } from "game_scene_boom/create_effect";
import { SpriteFrameModel } from  "pi_babylon/sprite/sprite_model";
import { PrefabModelObjPool } from "pi_babylon/index";
import { SpriteModel } from "pi_babylon/sprite/sprite_model";

export interface IBaseData {
    rayId: string;
    position: [number, number, number];
    scene: CustomScene;
    alphaIndex:number,
}

/**
 * 模型基类
 */
export abstract class ModleBase {

    protected modelRoot: IPrefab;
    protected rayId: string;

    constructor(data: IBaseData) {
        this.rayId = data.rayId;
        this.modelRoot = data.scene.prefabPool.createPrefab(`${data.rayId}_root`, NodeObj.PrefabInfo);
        this.modelRoot.setPosition(...data.position);
        this.modelRoot.setEnabled(true);
    }

    protected dispose() {
        this.modelRoot.dispose();
        this.modelRoot = undefined;
    }

    public get Root() {
        return this.modelRoot;
    }

    public get RayId() {
        return this.rayId;
    }
    /**创建精灵图 */
    public createSprite(icon: string, path: string, scaling: [number,number,number] = [1,1,1], alphaIndex: number = 1) {
        if (!PrefabModelObjPool.cfgMap.has(path)) {
            SpriteFrameModel.registPrefabCfg(path, "0", RES_PATH + path + ".png");
        }
        let model = this.data.scene.prefabPool.createPrefab(icon + ".png", path);
        model.setEnabled(true)
            .setScaling(...scaling)
            .setParent(this.modelRoot);
        model.alphaIndex = alphaIndex;
        model.resource.spriteFrame = icon + ".png";
        return model;
    }
    /**创建序列帧精灵图 */
    public createAnimaSprite(icon: string, path: string, scaling: [number,number,number] = [1,1,1], alphaIndex: number = 1) {
        if (!PrefabModelObjPool.cfgMap.has(path)) {
            SpriteModel.registPrefabCfg(this.data.scene.prefabPool, path, "0", RES_PATH + path + ".png");
        }
        let model = this.data.scene.prefabPool.createPrefab(icon + ".png", path);
        model.setEnabled(true)
            .setScaling(...scaling)
            .setParent(this.modelRoot);
        model.alphaIndex = alphaIndex;
        return model;
    }
    /**创建精灵图 */
    public createPsdSprite(icon: string, scaling: [number,number,number] = [1,1,1], alphaIndex: number = 1) {
        let model = CreateImgPlane.Instance.createImgPlaneForShow({
            rayId: "psd",
            scene: this.data.scene,
            icon: icon,
            cacheCount: 16,
            alphaIndex: alphaIndex,
            scaling,
            isEnabled: true,
        });
        model.setParent(this.modelRoot);
        return model;
    }
    /**创建特效 */
    public createU3dEffect(effName: string, animaName:string, scaling: [number,number,number] = [1, 1, 1], alphaIndex: number = 1) {
        let model = CreateEffect.Instance.createEffectForShow({
            path: `bahuang_eff/${effName}/`,
            fileName: effName,
            alphaIndex: alphaIndex,
            scaling,
            scene: this.data.scene,
            cacheCount: 16,
            isEnabled: true,
        });
        model.setParent(this.modelRoot);
        CreateEffect.Instance.changeAnim(model, animaName, true, 1);
        return model;
    }
}