import { ModelPrefab } from "pi_babylon/index";
import { ModleBase, IBaseData } from "./model_base";
import { DateMgr } from "pi_business_tool/date_mgr";
import { Anim } from 'game_scene_tween_anim/anim';

export class PickProgress extends ModleBase {
    //进度条远景背景蒙版
    public modelBgThumb: ModelPrefab;
    //进度条背景
    public modelBg: ModelPrefab;
    //进度条进度
    public modelLight: ModelPrefab;
    //进度条中间的图
    public modelIcon: ModelPrefab;

    //进度 0 -1
    private _progress: number;
    //进度条动画
    private tagId: number = 0;
    public constructor(public data: PickProgressData) {
        super(data);
        /**创建背景 */
        this.modelBgThumb = this.createSprite("thumb_bg", "bahuang_0", [80, 80, 1], this.data.alphaIndex + 1);
        this.modelBg = this.createSprite(this.data.bg, "bahuang_0", [64, 64, 1], this.data.alphaIndex + 1);
        this.modelLight = this.createSprite("progress", "progress", [58, 58, 1], this.data.alphaIndex + 2);
        this.modelIcon = this.createPsdSprite(this.data.icon, [30, -1 * 30, 1], this.data.alphaIndex + 2);

        let now:number = DateMgr.now();
        this.progress = (now - this.data.startTime)/(this.data.endTime - this.data.startTime);
        this.tagId = Number(this.data.rayId.split("-").shift());
        Anim.Instance.aphaFunASync(this.modelLight, this.tagId, this.data.endTime-now, this.progress, 1);
    }

    public get progress() {
        return this._progress;
    }
    //设置进度条进度
    public set progress(num: number) {
        this._progress = num;
        this.modelLight.setAlpha(num);
    }
    public setParent(root) {
        this.Root.setParent(root);
    }
    public setScale(num) {
        this.Root.setScale(num, num, num);
    }
    public scaleRender(showThumbnail:boolean) {
        this.modelBgThumb.setEnabled(showThumbnail);
    }

    //销毁
    public dispose() {
        this.modelBgThumb?.dispose();
        this.modelBg?.dispose();
        this.modelLight?.dispose();
        this.modelIcon?.dispose();
        this.modelBgThumb = undefined;
        this.modelBg = undefined;
        this.modelLight = undefined;
        this.modelIcon = undefined;
        Anim.Instance.clearTween(this.tagId);
        super.dispose();
    }
}

export interface PickProgressData extends IBaseData {
    bg:string; //进度条背景
    icon:string; //进度条中间显示
    endTime:number,//结束时间
    startTime:number,//开始时间
}


