import { ModelPrefab } from "pi_babylon/index";
import { ModleBase, IBaseData } from "./model_base";
import { Anim } from 'game_scene_tween_anim/anim';

//飞船头像显示
export class PlanetHead extends ModleBase {
    public modelIcon: ModelPrefab;
    public progressIcon: ModelPrefab;
    public killMoreIcon: ModelPrefab;
    private _killMore:boolean;
    private _animFun:Function;
    public constructor(public data: PlayerData) {
        super(data);
        this.modelIcon = this.createSprite(this.data.icon, "bahuang_dong_0", [100, -1 * 100, 1], this.data.alphaIndex);
        this.progressIcon = this.createSprite("progress_blood", "progress_blood", [100, 100, 1], this.data.alphaIndex + 1);
        this.progress = 0;
    }
    //设置进度条进度
    public set progress(num:number) {
        this.progressIcon.setAlpha(num);
    }
    //设置是血量动画
    public async startProgressAnima(progress:number, animTime:number) {
        Anim.Instance.clearTween(this.data.rayId);
        await Anim.Instance.aphaFunASync(this.progressIcon, this.data.rayId, animTime, 0, progress);
    }    

    //设置连胜
    public set killMore(bol:boolean) {
        if(this._killMore){
            return
        }
        this._killMore = bol;
        let model = this.createAnimaSprite("bahuang_anima2", "bahuang_anima2", [200, -200, 1], this.data.alphaIndex - 1);
        model.setAnim({
            isLoop: true,
            animName: "fight_win2",
            speed: 1,
        });
    }    
    //播放战斗失败动画
    public playFailAnima() {
        let model = this.createAnimaSprite("bahuang_anima2", "bahuang_anima2", [200, -200, 1], this.data.alphaIndex + 1);
        model.setAnim({
            isLoop: false,
            animName: "fight_fail",
            speed: 1,
        });
    }    
    //设置动画
    public get animFun() {
        return this._animFun;
    }    
    //设置动画方法
    public set animFun(fun:Function) {
        if(this.animFun){
            this.data.scene.impl.onBeforeRenderObservable.removeCallback(this.animFun);
        }
        this._animFun = fun;
        if(this._animFun){
            this.data.scene.impl.onBeforeRenderObservable.add(this.animFun);
        }
    }

    public setPosition(x, y, z) {
        this.Root.setPosition(x, y, z);
    }
    public setParent(root) {
        this.Root.setParent(root);
    }
    public setScale(num) {
        this.Root.setScale(num, num, num);
    }

    public scaleRender(showThumbnail:boolean) {
        this.modelIcon.setEnabled(!showThumbnail);
        this.progressIcon?.setEnabled(!showThumbnail);
        this.killMoreIcon?.setEnabled(!showThumbnail);
    }
    //销毁
    public dispose() {
        if(this.animFun){
            this.modelIcon.scene.impl.onBeforeRenderObservable.removeCallback(this.animFun);
        }
        this.modelIcon.dispose();
        this.killMoreIcon?.dispose();
        this.progressIcon?.dispose();
        this.modelIcon = undefined;
        this.killMoreIcon = undefined;
        this.progressIcon = undefined;
        super.dispose();
        Anim.Instance.clearTween(this.data.rayId);
    }
}



export interface PlayerData extends IBaseData {
    icon:string;//飞船
    name:string;//玩家名字
    progressTime:number;//动画时间
}

