import { ModelPrefab } from "pi_babylon/index";
import { ModleBase, IBaseData } from "./model_base";
import { Progress } from "./progress";
import { BABYLON } from "pi_babylon/render3d/babylon";
import { PointSign, SignData } from "./sign";
import { PickProgress } from "./pick_progress";
import { DateMgr } from "pi_business_tool/date_mgr";
import { ShowClickCeil } from "./ceil_click";
import { PointNum } from "./num";
import { PointEffect } from "./effect";
import { TimeoutPool } from "pi_babylon/util/timeout_pool";


//八荒地图点位混合显示
export class PointMix extends ModleBase {
    //建筑模型
    public model: ModelPrefab;
    //建筑特效
    public effectModel: PointEffect;
    //进度
    public progressModel: Progress | ShowClickCeil;
    //采集
    public pickModel: PickProgress;
    //类型标志
    public signModel: PointSign;
    //保护罩
    public protectModel: ModelPrefab;
    //点位状态
    public stateModel: ModelPrefab;
    //数量
    public modelNums: PointNum;
    public HSV_DISABLED:BABYLON.Color3 = new BABYLON.Color3(0, -1, 0);//灰色
    public HSV_NORMAL:BABYLON.Color3 = new BABYLON.Color3(0, 0, 0);//正常色

    public constructor(private data: PointData) {
        super(data);
        this.createEffect();
        this.createIcon();
        this.createProtect();
        this.createState();
        if(!data.isMyUnion || !data.icon){return}
        this.createNums(this.data.shikong as number);
        this.createPicking();
        this.createProgress();
        if(!this.data.pickData){
            this.createSign();
        }
    }

    //创建点位类型
    public createSign() {
        let signData = this.data.signData;
        if(signData){
            this.signModel = new PointSign({
                rayId: this.data.rayId + "_sign",
                position: [0, 90, 0],
                scene: this.data.scene,
                alphaIndex: this.data.alphaIndex,
                bg: signData.bg,
                icon: signData.icon,
                num: signData.num,
                star: this.data.star,
            });
            this.signModel.setParent(this.modelRoot);
            this.signModel.setScale(1.5);
            this.signModel.setHsi(this.data.lock ? this.HSV_DISABLED : this.HSV_NORMAL)
        }
    }
    //设置数字
    public createNums(num: number) {
        if(!(num > 0)){return }
        this.modelNums = new PointNum({
            rayId: this.data.rayId + "_num",
            bg: "black_bg",
            num,
            typeName: "num2_",
            space: 16,
            position:[0, -(this.data.height/2 + 15), 0],
            alphaIndex: this.data.alphaIndex + 2,
            scene: this.data.scene,
        });
        this.modelNums.setParent(this.modelRoot);
        this.modelNums.setScale(1.5);
    }
    //创建正在采集中
    public createPicking() {
        let pickData = this.data.pickData;
        if(pickData){
            this.pickModel = new PickProgress({
                rayId: this.data.rayId + "_pick",
                position: [0, 90, 0],
                scene: this.data.scene,
                alphaIndex: this.data.alphaIndex,
                bg: "pic_xh_jindu_bg",
                icon: pickData.icon,
                endTime: pickData.endTime,
                startTime: pickData.startTime,
            });
            this.pickModel.setParent(this.modelRoot);
            this.pickModel.setScale(1.2);
        }
    }
    //创建进度条
    public createProgress() {
        if(this.data.progressData){
            let data = this.calcProgess(this.data.progressData);
            if(data.progress < 1){//显示进度
                this.progressModel = new Progress({
                    rayId: this.data.rayId + "_progress",
                    position: [0, 90, 0],
                    scene: this.data.scene,
                    alphaIndex: this.data.alphaIndex,
                    bg: "pic_xh_jindu_bg",
                    icon: "pic_xh_jindu_suo",
                    progressData: data,
                });
            }else{//显示点击解锁
                this.progressModel = new ShowClickCeil({
                    rayId: this.data.rayId + "_complete",
                    position: [0, 90, 0],
                    scene: this.data.scene,
                    alphaIndex: this.data.alphaIndex,
                    icon: "pic_zjm_gongda",
                    textImg: "click_unlock",
                });
            }
            this.progressModel.setParent(this.modelRoot);
            this.progressModel.setScale(1.2);
        }
    }
    //计算进度条进度
    public calcProgess(data: ProgressInfo) {
        let progress = data.progress[0]/data.progress[1], animTime = 0;
        if(data.startTime > 0){
            let now = DateMgr.now();
            animTime = data.endTime - now;
            progress += (now - data.startTime) / (data.endTime - data.startTime);
        }
        return {
            progress,
            animTime,
        }
    }
    //创建点位图片
    public createIcon() {
        if(!this.data.icon){return}
        this.model = this.createSprite(this.data.icon, "bahuang_0", [this.data.width, this.data.height, 1], this.data.alphaIndex);
        let color = this.data.lock ? this.HSV_DISABLED : this.HSV_NORMAL;
        this.model.setColor(color.r, color.g, color.b);
    }
    //创建状态图片
    public createState() {
        if(this.data.state === PointState.FIGHT || this.data.state === PointState.ATT){
            this.model = this.createSprite("xuanzhan", "bahuang_0", [300, -300, 1], this.data.alphaIndex + 2);
            this.model.setPosition(300, 300, 0);
        }
    }
    //创建保护罩
    public createProtect() {
        let now = DateMgr.now();
        if(this.data.protectTime > now){
            this.protectModel = this.createAnimaSprite("bahuang_anima2", "bahuang_anima2", [600, -600, 1], this.data.alphaIndex + 2);
            this.protectModel.setAnim({
                isLoop: true,
                animName: "fight_win2",
                speed: 1,
            });
            TimeoutPool.setTimeout(()=>{
                this.protectModel?.dispose();
            }, this.data.protectTime - now);
        }
    }
    //创建点位特效
    public createEffect() {
        if(!this.data.effectList && !this.data.u3dEffect){return}
        this.effectModel = new PointEffect({
            rayId: this.data.rayId + "_effect",
            effectBg: this.data.effectBg,
            effectList: this.data.effectList,
            u3dEffect: this.data.u3dEffect,
            scene: this.data.scene,
            alphaIndex: this.data.alphaIndex,
            lock: this.data.state !== PointState.OWN,
            scaling:[this.data.width, -this.data.height, 1],
            position: [0, 0, 0],
        });
        this.effectModel.setParent(this.modelRoot);
    }

    public setPostion(x, y, z) {
        this.Root.setPosition(x, y, z);
    }
    //设置显示隐藏
    public scaleRender(showThumbnail:boolean) {
        //状态
        this.stateModel?.setEnabled(!showThumbnail);
        //保护罩
        this.protectModel?.setEnabled(!showThumbnail);
        //进度
        this.progressModel?.scaleRender(showThumbnail);
        //采集
        this.pickModel?.scaleRender(showThumbnail);
        //类型标志
        this.signModel?.scaleRender(showThumbnail);
        //时空之力
        this.modelNums?.Root?.setEnabled(showThumbnail);
    }

    //销毁
    public dispose() {
        this.model?.dispose();
        this.effectModel?.dispose();
        this.progressModel?.dispose();
        this.signModel?.dispose();
        this.pickModel?.dispose();
        this.modelNums?.dispose();
        this.protectModel?.dispose();
        this.stateModel?.dispose();
        this.model = undefined;
        this.effectModel = undefined;
        this.progressModel = undefined;
        this.signModel = undefined;
        this.pickModel = undefined;
        this.modelNums = undefined;
        this.protectModel = undefined;
        this.stateModel = undefined;
        super.dispose();
    }
}


interface PickData{
    icon:string; //中间显示
    endTime:number,//结束时间
    startTime:number,//开始时间
}
//解锁进度信息
interface ProgressInfo {
    progress: [number, number],//[已有值，需要值]
    startTime:number,//开始时间
    endTime:number,//结束时间
}

export interface PointData extends IBaseData {
    icon: string,//建筑图
    effectBg:string,//建筑特效背景
    effectList:string[],//建筑序列帧特效
    u3dEffect:string[],//建筑unity特效
    lock: number,
    width: number,
    height: number,
    isMyUnion: boolean,//是否我的联盟
    star:number,//星级
    shikong:number,//时空之力
    state:PointState,//点位状态
    protectTime:number,//保护时间
    progressData: ProgressInfo,//进度
    signData: SignData,
    pickData: PickData,
}

//状态
export enum PointState{
    OWN = 1,//占领
    FIGHT = 2,//战斗
    ATT = 3,//宣战
    NONE = 0,//不做处理
}