import { ModelPrefab } from "pi_babylon/index";
import { ModleBase, IBaseData } from "./model_base";
import { GlobalAtlasManager } from 'pi_babylon/sprite/sprite_atlas';
import { TextureProgressCircleMaterial } from "pi_babylon/material/texture_progress_circle";
import { RES_PATH } from "game_map/xls/map/map.struct";
import { Anim } from 'game_scene_tween_anim/anim';

export class Progress extends ModleBase {
    //进度条远景背景蒙版
    public modelBgThumb: ModelPrefab;
    //进度条背景
    public modelBg: ModelPrefab;
    //进度条进度
    public modelLight: ModelPrefab;
    //进度条中间的图
    public modelIcon: ModelPrefab;
    //进度条动画
    private tagId: number = 0;
    //进度 0 -1
    private progress: number;
    public constructor(public data: ProgressData) {
        super(data);
        /**创建背景 */
        this.modelBgThumb = this.createSprite( "thumb_bg", "bahuang_0", [100, 100, 1], this.data.alphaIndex);
        this.modelBg = this.createSprite(this.data.bg, "bahuang_0", [64, 64, 1], this.data.alphaIndex + 1);
        this.modelIcon = this.createSprite(this.data.icon, "bahuang_0", [24, -1 * 28, 1], this.data.alphaIndex + 2);
        this.modelLight = this.createSprite("progress", "progress", [58, 58, 1], this.data.alphaIndex + 2);
        this.progressData = data.progressData;
    }

    //设置进度条进度
    public set progressData(data: { progress:number, animTime:number}) {
        this.progress = data.progress;
        this.modelLight.setAlpha(this.progress);
        if(data.animTime > 0){
            this.tagId = Number(this.data.rayId.split("-").shift());
            Anim.Instance.aphaFunASync(this.modelLight, this.tagId, data.animTime, this.progress, 1);
        }
    }

    public setParent(root) {
        this.Root.setParent(root);
    }
    public setScale(num) {
        this.Root.setScale(num, num, num);
    }
    public scaleRender(showThumbnail:boolean) {
        this.modelBgThumb.setEnabled(showThumbnail);
    }

    //销毁
    public dispose() {
        this.modelBgThumb?.dispose();
        this.modelBg?.dispose();
        this.modelLight?.dispose();
        this.modelIcon?.dispose();
        this.modelBgThumb = undefined;
        this.modelLight = undefined;
        this.modelIcon = undefined;
        Anim.Instance.clearTween(this.tagId);
        super.dispose();
    }
}

export interface ProgressData extends IBaseData {
    bg:string; //进度条背景
    icon:string; //进度条中间显示
    progressData:{
        progress:number,//进度
        animTime:number,//动画时间
    }
}


// 圆形进度条注册
export const registImage = ()=>{
    GlobalAtlasManager.registImage(
        RES_PATH + "progress.png", "progress.png", 0, 0, 58, 58, 58, 58, undefined, undefined, undefined, undefined, 5, undefined
    );
    GlobalAtlasManager.registImage(
        RES_PATH + "progress_blood.png", "progress_blood.png", 0, 0, 196, 196, 196, 196, undefined, undefined, undefined, undefined, 5, undefined
    );
    TextureProgressCircleMaterial.RegistForSprite();
}

