import { ModelPrefab } from "pi_babylon/index";
import { ModleBase, IBaseData } from "./model_base";
import { PointNum } from "./num";
import { BABYLON } from "pi_babylon/render3d/babylon";
import { SpriteFrameModel } from  "pi_babylon/sprite/sprite_model";
import { PrefabModelObjPool } from "pi_babylon/index";

export class PointSign extends ModleBase {
    //背景
    public modelBg: ModelPrefab;
    //数量
    public modelNums: PointNum;
    //星级
    public modelStars: ModelPrefab[] = [];
    public _star: number;

    public constructor(private data: SignData) {
        super(data);
        /**创建背景 */
        this.modelBg = this.createSprite(this.data.bg + ".png", "bahuang_0", [64, -1 * 79, 1], this.data.alphaIndex);
        this.createNums(this.data.num as number);
        this.star = this.data.star;
    }

    //设置数字
    public createNums(num: number) {
        if(!(num > 0)){return }
        this.modelNums = new PointNum({
            rayId: this.data.rayId,
            num,
            typeName: "num_",
            space: 13,
            position:[0, -5, 0],
            alphaIndex: this.data.alphaIndex + 1,
            scene: this.data.scene,
        });
        this.modelNums.setParent(this.modelRoot);
    }
    //设置星级
    public set star(star: number) {
        this._star = star;
        this.disposeStars();
        if(this._star > 0){
            this.createStarSprite(this._star);
        }
    }
    /**创建星级*/
    public createStarSprite(count: number) {
        let x = 0;
        let diff = (count + 1)/2;
        while(count > 0){
            let _x = x + (diff - count) * 13;
            let model = this.createSprite(`icon_gouyu`, "bahuang_0", [19, -1 * 22, 1], this.data.alphaIndex + 1);
            model.setPosition(_x, 54, 0);
            this.modelStars.push(model);
            count--;
        }
    }
    public setParent(root) {
        this.Root.setParent(root);
    }
    public setScale(num) {
        this.Root.setScale(num, num, num);
    }
    public scaleRender(showThumbnail:number) {
        this.modelNums && this.modelNums.Root.setEnabled(!showThumbnail);
        this.modelBg.setEnabled(showThumbnail !== 2);
        this.modelStars.forEach(v=>v.setEnabled(showThumbnail !== 2));
        if(showThumbnail !== 2){
            this.modelBg.resource.spriteFrame = (showThumbnail ? this.data.bg + "2" : this.data.bg) + ".png";
            if(this.modelStars.length > 0){
                let x = 0;
                let diff = (this.modelStars.length + 1)/2;
                this.modelStars.forEach((v, i) =>{
                    let _x = x + (diff - (i + 1)) * 13;
        
                    if(showThumbnail){
                        v.resource.spriteFrame = "icon_gouyu_suoluetu.png";
                        v.setScale(13, -1 * 12, 1);
                        v.setPosition(_x, -32, 0);
                    }else{
                        v.resource.spriteFrame = "icon_gouyu.png";
                        v.setScale(19, -1 * 22, 1);
                        v.setPosition(_x, 54, 0);
                    }
                });
            }
        }
    }
    //设置hsi
    public setHsi(color:BABYLON.Color3) {
        this.modelBg && this.modelBg.setColor(color.r, color.g, color.b);
        this.modelStars && this.modelStars.forEach(v=>v.setColor(color.r, color.g, color.b));
    }

    //销毁
    public dispose() {
        this.modelBg?.dispose();
        this.modelNums?.dispose();
        this.modelStars.forEach(v=>v.dispose());
        this.modelBg = undefined;
        this.modelNums = undefined;
        this.modelStars.length = 0;
        super.dispose();
    }
    //销毁
    public disposeStars() {
        this.modelStars?.forEach(v=>v.dispose());
        this.modelStars.length = 0;
    }
}

export interface SignData extends IBaseData {
    bg:string; //背景
    icon:string; //中间显示
    star:number; //星级
    num?:number,//数量
}

